package host;

import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

//import javax.comm.PortInUseException;
//import javax.comm.UnsupportedCommOperationException;

/**
 * Recorder class, handles all recording and retrieving functions
 * @author bentzx
 *
 */
public class Recorder {

	private Panel panel_;
	private String folderPathName_;
	private boolean isRecording_ = false;
	
	/**
	 * Constructor
	 * @param p Panel
	 */
	public Recorder(Panel p){
		panel_ = p;
	}
	
	/**
	 * Start recording, capture the starting time 
	 */
	public void start(){
		isRecording_ = true;
		//create folder that has the name of rec start time
		folderPathName_ = "recordings\\" + getUniqueIdentifier();
		File f = new File(folderPathName_);
		f.mkdirs();
	}
	
	/**
	 * Store file in the record folder in bmp using filename of time in ms
	 * @param input The input file
	 */
	public void store(File input){
		//note takes bmp
		if (isRecording_){
			// create destination file
			File fileSave = new File(folderPathName_ + "\\" + System.currentTimeMillis() + ".bmp");
			try {
				boolean success = fileSave.createNewFile();
				if (success){
					copyFile(input, fileSave);	
				}
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
		}
	}
	
	/**
	 * Stop recording and update the playlist
	 */
	public void stop(){
		isRecording_ = false;
		panel_.updatePlaylist();
	}
	
	/**
	 * Retrieve recording and play
	 */
	public void retrieve(String playFolderName){
		File[] files = getRecFiles(playFolderName);
		if (files.length==0){
			panel_.getStatusTF().setText("Invalid recording");
		} else {
			int pctPlayed;
			for (int i=0; i<files.length; i++){
				if (panel_.getIsPlaying()){
					panel_.initDisplay(files[i]);
					pctPlayed = (int)Math.ceil((i+1)*100/files.length); //round up
					panel_.getStatusTF().setText("Played " + pctPlayed + "%");
					try {
						Thread.sleep(3000);
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				} else {
					break; //break out of play mode when stop button pressed
				}
			}
		}
	} 

	/**
	 * To get an array of folders/directories (as File) in the recordings directory 
	 * @return An array containing folders (File)
	 */
	protected File[] getRecFolders(){
		File dir = new File("recordings\\");
	    
	    // This filter only returns directories
	    FileFilter fileFilter = new FileFilter() {
	        public boolean accept(File file) {
	            return file.isDirectory();
	        }
	    };
	    File [] files = dir.listFiles(fileFilter);
	    
	    return files;
	}
	
	/**
	 * To get an array of files (bmp) in the specified folder in the recordings directory
	 * @param folderName The name of the folder in which to get files
	 * @return An array containing the files (bmp) in the specified folder
	 */
	private File[] getRecFiles(String folderName){
		File dir = new File("recordings\\" + folderName);
		File[] files = dir.listFiles();
		return files;
	}
	
	/**
	 * Delete the folder selected
	 * @param folderName The name of the folder for deletion
	 */
	protected void deleteFolder(String folderName){
		File dir = new File("recordings\\" + folderName);
		boolean delSuccessful = deleteDir(dir);
		if (delSuccessful){
			panel_.getStatusTF().setText("Delete successful");
		} else {
			panel_.getStatusTF().setText("Unable to delete");
		}
	}
	
	/**
	 * Recursively delete folders in the directory given
	 * @param dir Folder to be deleted
	 * @return A boolean if deletion is successful
	 */
	private boolean deleteDir(File dir) {
        if (dir.isDirectory()) {
            String[] children = dir.list();
            for (int i=0; i<children.length; i++) {
                boolean success = deleteDir(new File(dir, children[i]));
                if (!success) {
                    return false;
                }
            }
        }
    
        // The directory is now empty so delete it
        return dir.delete();
    }

	
	/**
	 * Generate a time based unique identifier for photo
	 * @return A string containing the unique identifier
	 */
	private String getUniqueIdentifier() {
        // Create a unique identifier based upon time.
        DateFormat format = new SimpleDateFormat("ddMMMyyyy_hhmmss_SSSa");
        System.out.println("check uniqueIdentifier" + format.format(new Date()));
        return format.format(new Date());
    }
	
    /**
     * Copy file from source file to destination file
     * @param src source file
     * @param dst destination file
     */
    private void copyFile(File src, File dst){
        try {
			InputStream in = new FileInputStream(src);
			OutputStream out = new FileOutputStream(dst);
   
			// Transfer bytes from in to out
			byte[] buf = new byte[1024];
			int len;
			while ((len = in.read(buf)) > 0) {
			    out.write(buf, 0, len);
			}
			in.close();
			out.close();
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }

	
//	public static void main(String [] argv) throws IOException, UnsupportedCommOperationException, PortInUseException {
//		//testing purposes
//		//Recorder rec = new Recorder();
//		Panel panel_ = new Panel(new SerialManager(new HostManager()));
//		Recorder rec = new Recorder(panel_);
//		rec.start();
//		rec.store(new File("recordings\\temp.bmp"));
//		rec.store(new File("recordings\\temp2.bmp"));
//		rec.store(new File("recordings\\picTest4.bmp"));
//		rec.stop();
//		rec.start();
//		rec.store(new File("recordings\\picTest5.bmp"));
//		rec.stop();
//		File [] files = rec.getRecFolders();
//		for (int i=0; i<files.length; i++){
//			System.out.println("folder names " + i + ": " + files[i].getName());
//			rec.retrieve(files[i].getName());
//		}
//		
//	}
}
