package host;


import java.io.IOException;
import java.util.TooManyListenersException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.Vector;
//import java.util.concurrent.LinkedBlockingQueue;

import gnu.io.CommPortIdentifier;
import gnu.io.CommPort;
import gnu.io.PortInUseException;
import gnu.io.SerialPort;
import gnu.io.SerialPortEvent;
import gnu.io.SerialPortEventListener;
import gnu.io.UnsupportedCommOperationException;

import javax.swing.JOptionPane;


/**
 * SerialManager class, handles all serial communication
 * @author bentzx
 *
 */
public class SerialManager implements SerialPortEventListener {

	private HostManager hostManager_;
	private SerialPort serialPort_ = null;
	private String comPort_ = null;
	private InputStream inputStream_ = null;
	private OutputStream outputStream_ = null;
//	private LinkedBlockingQueue <Integer> queue_ = null;
	
	//Serial port settings:
	private static final int PARITY = SerialPort.PARITY_NONE;
	private static final int STOPBITS = SerialPort.STOPBITS_2; //SerialPort.STOPBITS_1;
	private static final int BAUD = 38400; //38400, 57600, 115200
	private static final int DATABITS = SerialPort.DATABITS_8;
	
	/**
	 * Constructor
	 * @param hM HostManager
	 */
	public SerialManager(HostManager hM) {
		hostManager_ = hM;
	}
	
	/**
	 * To select a port
	 * @param msg The message to display in pop-up window
	 * @return The selected port
	 */
	private String selectPort(String msg) {
		Object[] list = listPorts(true);
		String port = null;
		if (list.length > 0) {
			port = (String) JOptionPane.showInputDialog(null, msg, "Input",
				JOptionPane.INFORMATION_MESSAGE, null, list, list[0]);
		}
		else {
			System.out.println("Must connect a serial port");
//			System.exit(0);
		}
		if (port == null) {
			System.out.println("Exiting");
//			System.exit(0); 
		} else {
			port = port.substring(0, port.indexOf(" - "));
			System.out.println(port);
		}
		
		return port;
	}
	
	/**
	 * Searches for serial ports
	 * @param available 
	 * @return The list of ports
	 */
	private Object[] listPorts(boolean available) {
		Vector <String> result = new Vector <String>();
		Enumeration portEnum = CommPortIdentifier.getPortIdentifiers();
		while (portEnum.hasMoreElements()) {
			CommPortIdentifier com = (CommPortIdentifier) portEnum
					.nextElement();
			if (available) {
				switch (com.getPortType()) {
				case CommPortIdentifier.PORT_SERIAL:
					try {
						// Test open the port to see if it's available for 50ms
						CommPort thePort = com.open("commtest", 50);
						thePort.close();
						result.add(new String(com.getName() + " - "
								+ getPortTypeName(com.getPortType())));
					} 
					catch (PortInUseException e) {
						System.err.println(e);
					} 
					catch (Exception e) {
						System.err.println(e);
					}
				}
			} else {
				result.add(new String(com.getName() + " - "
						+ getPortTypeName(com.getPortType())));
			}
		}
		return result.toArray();
	}
	
	/**
	 * Determines what type of port
	 * @param portType
	 * @return The type of port in String format
	 */
	private String getPortTypeName(int portType) {
		switch (portType) {
		case CommPortIdentifier.PORT_I2C:
			return "I2C";
		case CommPortIdentifier.PORT_PARALLEL:
			return "Parallel";
		case CommPortIdentifier.PORT_RAW:
			return "Raw";
		case CommPortIdentifier.PORT_RS485:
			return "RS485";
		case CommPortIdentifier.PORT_SERIAL:
			return "Serial";
		default:
			return "unknown type";
		}
	}
	
	/**
	 * Enables serial connection, user chooses COM port
	 * @return Boolean whether connection is successful
	 */
	protected boolean enableSerial(){
		System.out.println("Initializing serial... ");
		comPort_ = selectPort("Select Serial Port");
		if (comPort_==null){
			return false;
		} else {
			return converse();
		}
	}
	
	/**
	 * Sets up communication channel
	 * @return boolean whether connection is successful
	 */
	protected boolean converse() {
		CommPortIdentifier portId = null;
		Enumeration portList = null;
		boolean portFound = false;

		if (serialPort_ != null) {
			serialPort_.close();
		}
		// Get a list of available ports
		portList = CommPortIdentifier.getPortIdentifiers();
		while (portList.hasMoreElements() && !portFound) {
			portId = (CommPortIdentifier) portList.nextElement();
			
			if (portId.getPortType() == CommPortIdentifier.PORT_SERIAL) {
				// Tests whether this is the com port we want
				if (portId.getName().equals(comPort_)) {
					portFound = true;

					/* now put a thread to listen to the Serial port's events */
					try {
						if (serialPort_ != null) {
							serialPort_.close();
						}
						serialPort_ = (SerialPort) portId.open("videoCam port",
								10000); // wait up to 10 seconds for the port to open

						// set the Serial Port parameters
						serialPort_.setSerialPortParams(BAUD, DATABITS, STOPBITS, PARITY);

						// set flow control hardware to none
						serialPort_.setFlowControlMode(SerialPort.FLOWCONTROL_NONE);

						// get the streams associated with serial port
						inputStream_ = serialPort_.getInputStream();
						outputStream_ = serialPort_.getOutputStream();

						// start listening for events
						serialPort_.addEventListener(this);
						serialPort_.notifyOnDataAvailable(true);

					} catch (PortInUseException e) {
						System.out.println("Serial Port already in use.");
						System.err.println(e);
					} catch (IOException e) {
						System.out.println("An IO Exception occurred.");
						System.err.println(e);
					} catch (TooManyListenersException e) {
						System.out.println("too many listeners.");
						System.err.println(e);
					} catch (UnsupportedCommOperationException e) {
						System.out.println("Some Unknown Comm Exception Occurred.");
						System.err.println(e);
					}
				}
			}
		}
		if (!portFound) {
			System.out.println("Port " + comPort_ + " was not found!");
		}
		return portFound;
	}

	/* (non-Javadoc)
	 * @see gnu.io.SerialPortEventListener#serialEvent(gnu.io.SerialPortEvent)
	 */
	public void serialEvent(SerialPortEvent event) {
		switch (event.getEventType()) {

		case SerialPortEvent.BI:
		case SerialPortEvent.OE:
		case SerialPortEvent.FE:
		case SerialPortEvent.PE:
		case SerialPortEvent.CD:
		case SerialPortEvent.CTS:
		case SerialPortEvent.DSR:
		case SerialPortEvent.RI:
			System.out.println("Bad data received. Ignoring the most recent received data packet");
			break;
		case SerialPortEvent.OUTPUT_BUFFER_EMPTY:
			break;
		case SerialPortEvent.DATA_AVAILABLE:
			try {
				while (inputStream_.available() > 0) {
					int i = inputStream_.read();
					System.out.println("     " + intToHex(i));
					hostManager_.getDataAnalyzer().rxData(i);	// send data to dataAnalyzer for analysis
				}
			} 
			catch (IOException e) {
				System.err.println("IO Exception occurred while trying to read the data from inputStream.");
				System.err.println(e);
			}
			break;
		}
	}
	
	/**
	 * Disconnects serial connection
	 * @return Boolean false for disconnected serial
	 */
	public boolean disconnect() {
		serialPort_.close();
		return false; // false for disconnection
	}
	
	/**
	 * Automatically disconnects and reconnects serial with previous settings
	 */
	public void autoReconnect(){
		AutoReconnectThread aRecon = new AutoReconnectThread(this); //use new thread, otherwise program freezes
		aRecon.start();
	}
	
//	 public static void main(String[] argv){
//	    new SerialManager(null).converse();
//	  }
	
	/**
	 * Converts int to hex
	 * @param i Int to be converted
	 * @return Hex in String format
	 */
	private String intToHex(int i) {
		//String hex = Integer.toHexString(i);
		String hex = Integer.toHexString(0x100 | (0x0ff & i)).substring(1);
		if (hex.length() == 1) {
			hex = "0" + hex;
		}
		return hex;
	}
}

/**
 * Requires this thread for auto reconnect, otherwise, program freezes
 * @author bentzx
 *
 */
class AutoReconnectThread extends Thread {
	private SerialManager serialManager_;
	
	public AutoReconnectThread(SerialManager sM){
		serialManager_ = sM;
	}

	public void run() {
		System.out.println("Auto-reconnecting");
		serialManager_.converse();
	}
}
